==> unity-ads-4.12.0-sources.jar: com/unity3d/services/core/properties/SdkProperties.java <==
package com.unity3d.services.core.properties;

import static com.unity3d.services.core.misc.Utilities.wrapCustomerListener;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.util.Base64;

import com.unity3d.ads.BuildConfig;
import com.unity3d.ads.IUnityAdsInitializationListener;
import com.unity3d.ads.UnityAds;
import com.unity3d.services.core.cache.CacheDirectory;
import com.unity3d.services.core.configuration.Configuration;
import com.unity3d.services.core.device.Device;
import com.unity3d.services.core.log.DeviceLog;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.LinkedHashSet;
import java.util.concurrent.atomic.AtomicReference;

public class SdkProperties {
	private static String _configUrl = null;
	private static CacheDirectory _cacheDirectory = null;
	private static final String CACHE_DIR_NAME = "UnityAdsCache";
	private static final String LOCAL_CACHE_FILE_PREFIX = "UnityAdsCache-";
	private static final String LOCAL_STORAGE_FILE_PREFIX = "UnityAdsStorage-";
	private static final String CHINA_ISO_ALPHA_2_CODE = "CN";
	private static final String CHINA_ISO_ALPHA_3_CODE = "CHN";
	private static final String DEFAULT_CONFIG_VERSION = "configv2";
	/**
	 * CHINA_CONFIG_HOSTNAME is Base64 Encoded to remove
	 * direct links to China country code endpoints within the code
	 */
	private static final String CHINA_CONFIG_HOSTNAME = "dW5pdHlhZHMudW5pdHljaGluYS5jbg==";
	private static final String DEFAULT_CONFIG_HOSTNAME = "unityads.unity3d.com";
	private static final String CONFIG_VERSION_METADATA_KEY = "com.unity3d.ads.configversion";
	private static long _initializationTime = 0;
	private static long _initializationTimeEpochMs = 0;
	private static long _appInitializationTimeEpochMs = 0;
	private static Configuration _latestConfiguration;

	private static final LinkedHashSet<IUnityAdsInitializationListener> _initializationListeners = new LinkedHashSet<IUnityAdsInitializationListener>();

	private static boolean _initialized = false;
	private static boolean _reinitialized = false;
	private static boolean _testMode = false;
	private static boolean _debugMode = false;
	private static final AtomicReference<InitializationState> _currentInitializationState = new AtomicReference<>(InitializationState.NOT_INITIALIZED);

	public enum InitializationState {
		NOT_INITIALIZED,
		INITIALIZING,
		INITIALIZED_SUCCESSFULLY,
		INITIALIZED_FAILED
	}

	public static void notifyInitializationFailed(UnityAds.UnityAdsInitializationError error, String message) {
		setInitializeState(InitializationState.INITIALIZED_FAILED);

		for(IUnityAdsInitializationListener initializationListener: getInitializationListeners()) {
			wrapCustomerListener(() -> initializationListener.onInitializationFailed(error, message));
		}
		resetInitializationListeners();
	}

	public static void notifyInitializationComplete() {
		setInitializeState(InitializationState.INITIALIZED_SUCCESSFULLY);

		for(IUnityAdsInitializationListener initializationListener: getInitializationListeners()) {
			wrapCustomerListener(initializationListener::onInitializationComplete);
		}
		resetInitializationListeners();
	}

	public static void setInitializeState(InitializationState initializeState) {
		_currentInitializationState.set(initializeState);
	}

	public static InitializationState getCurrentInitializationState() {
		return _currentInitializationState.get();
	}

	public static boolean isInitialized() {
		return _initialized;
	}

	public static void setInitialized(boolean initialized) {
		_initialized = initialized;
	}

	public static boolean isTestMode() {
		return _testMode;
	}

	public static void setTestMode(boolean testMode) {
		_testMode = testMode;
	}

	public static int getVersionCode () {
		return BuildConfig.VERSION_CODE;
	}

	public static String getVersionName() {
		return BuildConfig.VERSION_NAME;
	}

	public static String getCacheDirectoryName() {
		return CACHE_DIR_NAME;
	}

	public static String getCacheFilePrefix() {
		return LOCAL_CACHE_FILE_PREFIX;
	}

	public static String getLocalStorageFilePrefix() {
		return LOCAL_STORAGE_FILE_PREFIX;
	}

	public static void setConfigUrl(String url) throws URISyntaxException, MalformedURLException {
		if (url == null) {
			throw new MalformedURLException();
		}
		if (!url.startsWith("http://") && !url.startsWith("https://")) {
			throw new MalformedURLException();
		}

		URL u = new URL(url);
		u.toURI();

		_configUrl = url;
	}

	public static String getConfigUrl() {
		if (_configUrl == null) {
			_configUrl = getDefaultConfigUrl("release");
		}
		return _configUrl;
	}

	public static String getDefaultConfigUrl(String flavor) {
		boolean isChinaLocale = isChinaLocale(Device.getNetworkCountryISO());
		String hostName = isChinaLocale ? new String(Base64.decode(CHINA_CONFIG_HOSTNAME, Base64.DEFAULT)) : DEFAULT_CONFIG_HOSTNAME;
		return "https://" + getConfigVersion(ClientProperties.getApplicationContext()) + '.' + hostName + "/webview/" + getWebViewBranch() + "/" + flavor + "/config.json";
	}

	public static String getConfigVersion(Context context) {
		String configVersion = DEFAULT_CONFIG_VERSION;

		try {
			if (context != null) {
				ApplicationInfo app = context.getPackageManager().getApplicationInfo(context.getPackageName(), PackageManager.GET_META_DATA);
				Bundle bundle = app.metaData;
				if (bundle != null) {
					configVersion = bundle.getString(CONFIG_VERSION_METADATA_KEY, DEFAULT_CONFIG_VERSION);
				}
			}
		} catch (PackageManager.NameNotFoundException e) {
			DeviceLog.warning("Failed to retrieve application info for current package");
		}

		return configVersion;
	}

	private static String getWebViewBranch() {
		if (BuildConfig.DEBUG) {
			return BuildConfig.WEBVIEW_BRANCH;
		}
		return getVersionName();
	}

	public static String getLocalWebViewFile() {
		if (getCacheDirectory() == null) return "";
		return getCacheDirectory().getAbsolutePath() + "/" + "UnityAdsWebApp.html";
	}

	public static String getLocalConfigurationFilepath() {
		if (getCacheDirectory() == null) return "";
		return getCacheDirectory().getAbsolutePath() + "/" + "UnityAdsWebViewConfiguration.json";
	}

	public static void setLatestConfiguration(Configuration configuration) {
		_latestConfiguration = configuration;
	}

	public static Configuration getLatestConfiguration() {
		return _latestConfiguration;
	}

	public static File getCacheDirectory() {
		return getCacheDirectory(ClientProperties.getApplicationContext());
	}

	public static File getCacheDirectory(Context context) {
		if (_cacheDirectory == null) {
			setCacheDirectory(new CacheDirectory(CACHE_DIR_NAME));
		}

		return _cacheDirectory.getCacheDirectory(context);
	}

	public static void setCacheDirectory(CacheDirectory cacheDirectory) {
		_cacheDirectory = cacheDirectory;
	}

	public static CacheDirectory getCacheDirectoryObject() {
		return _cacheDirectory;
	}

	public static void setInitializationTime(long milliseconds) {
		_initializationTime = milliseconds;
	}

	public static long getInitializationTime() {
		return _initializationTime;
	}

	public static void setAppInitializationTimeSinceEpoch(long milliseconds) {
		_appInitializationTimeEpochMs = milliseconds;
	}

	public static long getAppInitializationTimeSinceEpoch() {
		return _appInitializationTimeEpochMs;
	}

	public static void setInitializationTimeSinceEpoch(long initializationTimeEpochMs) {
		_initializationTimeEpochMs = initializationTimeEpochMs;
	}

	public static long getInitializationTimeEpoch() {
		return _initializationTimeEpochMs;
	}

	public static void setReinitialized(boolean status) {
		_reinitialized = status;
	}

	public static boolean isReinitialized() {
		return _reinitialized;
	}

	public static void setDebugMode(boolean debugMode) {
		_debugMode = debugMode;

		if (debugMode) {
			DeviceLog.setLogLevel(DeviceLog.LOGLEVEL_DEBUG);
		} else {
			DeviceLog.setLogLevel(DeviceLog.LOGLEVEL_INFO);
		}
	}

	public static boolean getDebugMode() {
		return _debugMode;
	}

	public static void addInitializationListener(IUnityAdsInitializationListener listener) {
		if(listener == null) {
			return;
		}

		synchronized (_initializationListeners) {
			_initializationListeners.add(listener);
		}
	}

	public static IUnityAdsInitializationListener[] getInitializationListeners() {
		synchronized (_initializationListeners) {
			IUnityAdsInitializationListener[] listeners = new IUnityAdsInitializationListener[_initializationListeners.size()];
			_initializationListeners.toArray(listeners);
			return listeners;
		}
	}

	public static void resetInitializationListeners () {
		synchronized (_initializationListeners) {
			_initializationListeners.clear();
		}
	}

	public static boolean isChinaLocale(String networkISOCode) {
		if (networkISOCode.equalsIgnoreCase(CHINA_ISO_ALPHA_2_CODE)
			|| networkISOCode.equalsIgnoreCase(CHINA_ISO_ALPHA_3_CODE)) {
			return true;
		} else {
			return false;
		}
	}
}

==> unity-ads-4.12.4-sources.jar: com/unity3d/services/core/properties/SdkProperties.java <==
package com.unity3d.services.core.properties;

import static com.unity3d.services.core.misc.Utilities.wrapCustomerListener;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.util.Base64;

import com.unity3d.ads.BuildConfig;
import com.unity3d.ads.IUnityAdsInitializationListener;
import com.unity3d.ads.UnityAds;
import com.unity3d.services.core.cache.CacheDirectory;
import com.unity3d.services.core.configuration.Configuration;
import com.unity3d.services.core.device.Device;
import com.unity3d.services.core.log.DeviceLog;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.LinkedHashSet;
import java.util.concurrent.atomic.AtomicReference;

public class SdkProperties {
	private static String _configUrl = null;
	private static CacheDirectory _cacheDirectory = null;
	private static CacheDirectory _webviewCacheDirectory = null;
	private static final String CACHE_DIR_NAME = "UnityAdsCache";
	private static final String WEBVIEW_CACHE_DIR_NAME = "UnityAdsWebViewCache";
	private static final String LOCAL_CACHE_FILE_PREFIX = "UnityAdsCache-";
	private static final String LOCAL_STORAGE_FILE_PREFIX = "UnityAdsStorage-";
	private static final String CHINA_ISO_ALPHA_2_CODE = "CN";
	private static final String CHINA_ISO_ALPHA_3_CODE = "CHN";
	private static final String DEFAULT_CONFIG_VERSION = "configv2";
	/**
	 * CHINA_CONFIG_HOSTNAME is Base64 Encoded to remove
	 * direct links to China country code endpoints within the code
	 */
	private static final String CHINA_CONFIG_HOSTNAME = "dW5pdHlhZHMudW5pdHljaGluYS5jbg==";
	private static final String DEFAULT_CONFIG_HOSTNAME = "unityads.unity3d.com";
	private static final String CONFIG_VERSION_METADATA_KEY = "com.unity3d.ads.configversion";
	private static long _initializationTime = 0;
	private static long _initializationTimeEpochMs = 0;
	private static long _appInitializationTimeEpochMs = 0;
	private static Configuration _latestConfiguration;

	private static final LinkedHashSet<IUnityAdsInitializationListener> _initializationListeners = new LinkedHashSet<IUnityAdsInitializationListener>();

	private static boolean _initialized = false;
	private static boolean _reinitialized = false;
	private static boolean _testMode = false;
	private static boolean _debugMode = false;
	private static final AtomicReference<InitializationState> _currentInitializationState = new AtomicReference<>(InitializationState.NOT_INITIALIZED);

	public enum InitializationState {
		NOT_INITIALIZED,
		INITIALIZING,
		INITIALIZED_SUCCESSFULLY,
		INITIALIZED_FAILED
	}

	public static void notifyInitializationFailed(UnityAds.UnityAdsInitializationError error, String message) {
		setInitializeState(InitializationState.INITIALIZED_FAILED);

		for(IUnityAdsInitializationListener initializationListener: getInitializationListeners()) {
			wrapCustomerListener(() -> initializationListener.onInitializationFailed(error, message));
		}
		resetInitializationListeners();
	}

	public static void notifyInitializationComplete() {
		setInitializeState(InitializationState.INITIALIZED_SUCCESSFULLY);

		for(IUnityAdsInitializationListener initializationListener: getInitializationListeners()) {
			wrapCustomerListener(initializationListener::onInitializationComplete);
		}
		resetInitializationListeners();
	}

	public static void setInitializeState(InitializationState initializeState) {
		_currentInitializationState.set(initializeState);
	}

	public static InitializationState getCurrentInitializationState() {
		return _currentInitializationState.get();
	}

	public static boolean isInitialized() {
		return _initialized;
	}

	public static void setInitialized(boolean initialized) {
		_initialized = initialized;
	}

	public static boolean isTestMode() {
		return _testMode;
	}

	public static void setTestMode(boolean testMode) {
		_testMode = testMode;
	}

	public static int getVersionCode () {
		return BuildConfig.VERSION_CODE;
	}

	public static String getVersionName() {
		return BuildConfig.VERSION_NAME;
	}

	public static String getCacheDirectoryName() {
		return CACHE_DIR_NAME;
	}

	public static String getCacheFilePrefix() {
		return LOCAL_CACHE_FILE_PREFIX;
	}

	public static String getLocalStorageFilePrefix() {
		return LOCAL_STORAGE_FILE_PREFIX;
	}

	public static void setConfigUrl(String url) throws URISyntaxException, MalformedURLException {
		if (url == null) {
			throw new MalformedURLException();
		}
		if (!url.startsWith("http://") && !url.startsWith("https://")) {
			throw new MalformedURLException();
		}

		URL u = new URL(url);
		u.toURI();

		_configUrl = url;
	}

	public static String getConfigUrl() {
		if (_configUrl == null) {
			_configUrl = getDefaultConfigUrl("release");
		}
		return _configUrl;
	}

	public static String getDefaultConfigUrl(String flavor) {
		boolean isChinaLocale = isChinaLocale(Device.getNetworkCountryISO());
		String hostName = isChinaLocale ? new String(Base64.decode(CHINA_CONFIG_HOSTNAME, Base64.DEFAULT)) : DEFAULT_CONFIG_HOSTNAME;
		return "https://" + getConfigVersion(ClientProperties.getApplicationContext()) + '.' + hostName + "/webview/" + getWebViewBranch() + "/" + flavor + "/config.json";
	}

	public static String getConfigVersion(Context context) {
		String configVersion = DEFAULT_CONFIG_VERSION;

		try {
			if (context != null) {
				ApplicationInfo app = context.getPackageManager().getApplicationInfo(context.getPackageName(), PackageManager.GET_META_DATA);
				Bundle bundle = app.metaData;
				if (bundle != null) {
					configVersion = bundle.getString(CONFIG_VERSION_METADATA_KEY, DEFAULT_CONFIG_VERSION);
				}
			}
		} catch (PackageManager.NameNotFoundException e) {
			DeviceLog.warning("Failed to retrieve application info for current package");
		}

		return configVersion;
	}

	private static String getWebViewBranch() {
		if (BuildConfig.DEBUG) {
			return BuildConfig.WEBVIEW_BRANCH;
		}
		return getVersionName();
	}

	public static String getLocalWebViewFile() {
		if (getWebViewCacheDirectory() == null) return "";
		return getWebViewCacheDirectory().getAbsolutePath() + "/" + "UnityAdsWebApp.html";
	}

	public static String getLocalConfigurationFilepath() {
		if (getWebViewCacheDirectory() == null) return "";
		return getWebViewCacheDirectory().getAbsolutePath() + "/" + "UnityAdsWebViewConfiguration.json";
	}

	public static void setLatestConfiguration(Configuration configuration) {
		_latestConfiguration = configuration;
	}

	public static Configuration getLatestConfiguration() {
		return _latestConfiguration;
	}

	public static File getWebViewCacheDirectory() {
		return getWebViewCacheDirectory(ClientProperties.getApplicationContext());
	}

	public static File getWebViewCacheDirectory(Context context) {
		if (_webviewCacheDirectory == null) {
			setWebViewCacheDirectory(new CacheDirectory(WEBVIEW_CACHE_DIR_NAME, false));
		}

		return _webviewCacheDirectory.getCacheDirectory(context);
	}

	public static void setWebViewCacheDirectory(CacheDirectory cacheDirectory) {
		_webviewCacheDirectory = cacheDirectory;
	}

	public static File getCacheDirectory() {
		return getCacheDirectory(ClientProperties.getApplicationContext());
	}

	public static File getCacheDirectory(Context context) {
		if (_cacheDirectory == null) {
			setCacheDirectory(new CacheDirectory(CACHE_DIR_NAME));
		}

		return _cacheDirectory.getCacheDirectory(context);
	}

	public static void setCacheDirectory(CacheDirectory cacheDirectory) {
		_cacheDirectory = cacheDirectory;
	}

	public static CacheDirectory getCacheDirectoryObject() {
		return _cacheDirectory;
	}

	public static void setInitializationTime(long milliseconds) {
		_initializationTime = milliseconds;
	}

	public static long getInitializationTime() {
		return _initializationTime;
	}

	public static void setAppInitializationTimeSinceEpoch(long milliseconds) {
		_appInitializationTimeEpochMs = milliseconds;
	}

	public static long getAppInitializationTimeSinceEpoch() {
		return _appInitializationTimeEpochMs;
	}

	public static void setInitializationTimeSinceEpoch(long initializationTimeEpochMs) {
		_initializationTimeEpochMs = initializationTimeEpochMs;
	}

	public static long getInitializationTimeEpoch() {
		return _initializationTimeEpochMs;
	}

	public static void setReinitialized(boolean status) {
		_reinitialized = status;
	}

	public static boolean isReinitialized() {
		return _reinitialized;
	}

	public static void setDebugMode(boolean debugMode) {
		_debugMode = debugMode;

		if (debugMode) {
			DeviceLog.setLogLevel(DeviceLog.LOGLEVEL_DEBUG);
		} else {
			DeviceLog.setLogLevel(DeviceLog.LOGLEVEL_INFO);
		}
	}

	public static boolean getDebugMode() {
		return _debugMode;
	}

	public static void addInitializationListener(IUnityAdsInitializationListener listener) {
		if(listener == null) {
			return;
		}

		synchronized (_initializationListeners) {
			_initializationListeners.add(listener);
		}
	}

	public static IUnityAdsInitializationListener[] getInitializationListeners() {
		synchronized (_initializationListeners) {
			IUnityAdsInitializationListener[] listeners = new IUnityAdsInitializationListener[_initializationListeners.size()];
			_initializationListeners.toArray(listeners);
			return listeners;
		}
	}

	public static void resetInitializationListeners () {
		synchronized (_initializationListeners) {
			_initializationListeners.clear();
		}
	}

	public static boolean isChinaLocale(String networkISOCode) {
		if (networkISOCode.equalsIgnoreCase(CHINA_ISO_ALPHA_2_CODE)
			|| networkISOCode.equalsIgnoreCase(CHINA_ISO_ALPHA_3_CODE)) {
			return true;
		} else {
			return false;
		}
	}
}
